@@ -33,6 +33,8 @@
 #' named 'raw'. In case no object is provided, cNORM uses the inbuilt sample data to
 #' demonstrate the procedure.
 #' @param group grouping variable in the data, e. g. age groups, grades ...
+#' Setting group = FALSE deactivates modelling in dependence of age. Use this in case you do want
+#' conventional norm tables.
 #' @param raw the raw scores
 #' @param age the continuous explanatory variable; by default set to "group"
 #' @param width if a width is provided, the function switches to rankBySlidingWindow to determine the
@@ -48,6 +50,11 @@
 #'
 #' # variable names can be specified as well, here with the BMI data included in the package
 #' data.bmi <- prepareData(CDC, group="group", raw="bmi", age="age")
+#'
+#' # modelling with only one group with the 'elfe' dataset as an example
+#' # this results in conventional norming
+#' data.elfe2 <- prepareData(data = elfe, group = FALSE)
+#' m <- bestModel(data.elfe2)
 #' @export
 prepareData <- function(data = NULL, group = "group", raw = "raw", age = "group", width = NA, scale = "T") {
   if (is.null(data)) {
@@ -57,7 +64,7 @@ prepareData <- function(data = NULL, group = "group", raw = "raw", age = "group"
   }
 
   # checks
-  if (!(group %in% colnames(normData))) {
+  if ((typeof(group) != "logical") && !(group %in% colnames(normData))) {
     stop(paste(c("ERROR: Grouping variable '", group, "' does not exist in data object."), collapse = ""))
   } else if (!(raw %in% colnames(normData))) {
     stop(paste(c("ERROR: Raw score variable '", raw, "' does not exist in data object."), collapse = ""))
@@ -65,7 +72,7 @@ prepareData <- function(data = NULL, group = "group", raw = "raw", age = "group"
     stop(paste(c("ERROR: Age variable '", age, "' does not exist in data object."), collapse = ""))
   }
 
-  if (!is.numeric(normData[, group])) {
+  if ((typeof(group) != "logical") && !is.numeric(normData[, group])) {
     warning(paste(c("Grouping variable '", group, "' has to be numeric."), collapse = ""))
   }
 
@@ -79,9 +86,11 @@ prepareData <- function(data = NULL, group = "group", raw = "raw", age = "group"
 
   # exclude missings
   normData <- as.data.frame(normData)
-  normData <- normData[!is.na(normData[, group]), ]
+  if(typeof(group) != "logical"){
+    normData <- normData[!is.na(normData[, group]), ]
+    normData <- normData[!is.na(normData[, age]), ]
+  }
   normData <- normData[!is.na(normData[, raw]), ]
-  normData <- normData[!is.na(normData[, age]), ]
 
   # ranking and powers
   if(is.na(width)){
@@ -89,7 +98,12 @@ prepareData <- function(data = NULL, group = "group", raw = "raw", age = "group"
   }else{
     normData <- rankBySlidingWindow(normData, group = group, raw = raw, width = width, scale = scale)
   }
-  normData <- computePowers(normData, k = 4, norm = "normValue", age = age)
+
+  if(typeof(group) != "logical"||group)
+    normData <- computePowers(normData, k = 4, norm = "normValue", age = age)
+  else
+    normData <- computePowers(normData, k = 4, norm = "normValue")
+
   return(normData)
 }
 
@@ -124,12 +138,18 @@ prepareData <- function(data = NULL, group = "group", raw = "raw", age = "group"
 #' @return the dataset with the percentiles and norm scales per group
 #'
 #' @examples
-#' #Transformation with default parameters: RandIt and converting to T scores
+#' # Transformation with default parameters: RandIt and converting to T scores
 #' data.elfe <- rankByGroup(elfe, group = "group")
 #'
-#' #Transformation into Wechsler scores with Yu & Huang (2001) ranking procedure
+#' # Transformation into Wechsler scores with Yu & Huang (2001) ranking procedure
 #' data.elfe <- rankByGroup(elfe, group = "group", method = 7, scale=c(10, 3))
 #'
+#' # cNORM can as well be used for conventional norming
+#' # The group variable has to be set to NULL when ranking the group in this case
+#' d <- rankByGroup(elfe, raw="raw", group=FALSE)
+#' d <- computePowers(d)
+#' m <- bestModel(d)
+#' rawTable(0, model = m) # please use an arbitrary value for age when generating the tables
 #' @seealso rankBySlidingWindow, computePowers
 #' @export
 rankByGroup <-
@@ -157,7 +177,7 @@ rankByGroup <-
       stop(paste(c("ERROR: Raw value variable '", raw, "' does not exist in data object."), collapse = ""))
     }
 
-    if (!is.numeric(d[, group])) {
+    if ((typeof(group) != "logical") && !is.numeric(d[, group])) {
       warning(paste(c("Grouping variable '", group, "' has to be numeric."), collapse = ""))
     }
 
@@ -269,8 +289,8 @@ rankByGroup <-
 #' data sample, the sliding stops and the sample is drawn from the interval min + width and
 #' max - width, respectively.
 #' @param data data.frame with norm sample data
-#' @param age the continuous age variable
-#' group to FALSE cancels grouping (data is treated as one group)
+#' @param age the continuous age variable. Setting 'age' to FALSE inhibits computation of
+#' powers of age and the interactions
 #' @param raw name of the raw value variable (default 'raw')
 #' @param width the width of the sliding window
 #' @param method Ranking method in case of bindings, please provide an index,
@@ -494,27 +514,33 @@ computePowers <-
       norm <- attr(d, "normValue")
     }
 
+    useAge <- TRUE
+
     if (is.null(age)) {
       age <- attr(d, "age")
     }
 
+    if((typeof(age) == "logical") && !age){
+      useAge <- FALSE
+    }
+
     # check if columns exist
     if (!(norm %in% colnames(d))) {
       stop(paste(c("ERROR: Norm variable '", norm, "' does not exist in data object."), collapse = ""))
     }
 
-    if (!(age %in% colnames(d))) {
-      stop(paste(c("ERROR: Explanatory variable '", age, "' does not exist in data object."), collapse = ""))
-    }
-
     if (!is.numeric(d[, norm])) {
       warning(paste(c("Norm score variable '", norm, "' has to be numeric."), collapse = ""))
     }
 
-    if (!is.numeric(d[, age])) {
+    if (useAge && !is.numeric(d[, age])) {
       warning(paste(c("Age variable '", age, "' has to be numeric."), collapse = ""))
     }
 
+    if (useAge && !(age %in% colnames(d))) {
+      stop(paste(c("ERROR: Explanatory variable '", age, "' does not exist in data object."), collapse = ""))
+    }
+
     if ((k < 1) | (k > 6)) {
       message("Parameter k out of range, setting to 4")
       k <- 6
@@ -522,12 +548,13 @@ computePowers <-
 
 
     L1 <- as.numeric(d[[norm]])
-    A1 <- as.numeric(d[[age]])
-    L1A1 <- L1 * A1
-
     d$L1 <- L1
-    d$A1 <- A1
-    d$L1A1 <- L1A1
+
+    if(useAge){
+      A1 <- as.numeric(d[[age]])
+      L1A1 <- L1 * A1
+      d$A1 <- A1
+      d$L1A1 <- L1A1
 
     if (k > 1) {
       d$L2 <- d$L1 * d$L1
@@ -590,11 +617,24 @@ computePowers <-
       d$L6A5 <- d$L6 * d$A5
       d$L6A6 <- d$L6 * d$A6
     }
+    }else{
+      if (k > 1)
+        d$L2 <- d$L1 * d$L1
+      if (k > 2)
+        d$L3 <- d$L2 * d$L1
+      if (k > 3)
+        d$L4 <- d$L3 * d$L1
+      if (k > 4)
+        d$L5 <- d$L4 * d$L1
+      if (k > 5)
+        d$L6 <- d$L5 * d$L1
+    }
 
     # attributes
     attr(d, "age") <- age
     attr(d, "normValue") <- norm
     attr(d, "k") <- k
+    attr(d, "useAge") <- useAge
 
     return(d)
   }
